/* This file is an image processing operation for GEGL
 *
 * GEGL is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * GEGL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
 *
 * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
 * 2023 Beaver, Ringed Bevel
You can test plugin without installing by pasting this syntax into Gimp's GEGL graph (This graph requires other plugins 'ssg' and 'custom-bevel' this filter ships with those two plugins)

levels out-low=0.00
id=1
dst-out aux=[ ref=1

median-blur radius=5 alpha-percentile=0
ssg radius=0 stroke=7
ssg radius=0 stroke=7
ssg radius=0 stroke=7
ssg radius=0 stroke=7

median-blur radius=0
 ]
custom-bevel depth=4 gaus=1
dst-over aux=[ ref=1 custom-bevel depth=1 gaus=1  ]
 */


#include "config.h"
#include <glib/gi18n-lib.h>

#ifdef GEGL_PROPERTIES

property_boolean(advanced_options, _("Detailed options"), FALSE)
   description  (_("Show all settings for this plugin."))


enum_start (ring_time)
  enum_value (onering,      "onering",
              N_("One Ring"))
  enum_value (tworings,      "tworings",
              N_("Two Rings"))
  enum_value (threerings,      "threerings",
              N_("Three Rings"))
  enum_value (fourrings,      "fourrings",
              N_("Four Rings"))
enum_end (ringtime)
/* This ENUM list has a unique name to avoid conflicts with other plugins or Gimp filters*/

property_enum (rings, _("Amount of Rings"),
    ringtime, ring_time,
    fourrings)
   description  (_("Choose the amount of rings 1-4"))


property_int  (size, _("Size of the Bevel"), 3)
  value_range (1, 9)
  ui_range    (1, 9)
  ui_meta     ("unit", "pixel-distance")
  description (_("Median Radius to control the size of the bevel"))


property_double (azimuth, _("Azimuth"), 67.0)
    description (_("Light angle (degrees)"))
    value_range (20, 90)
    ui_meta ("unit", "degree")
    ui_meta ("direction", "ccw")

property_double (elevation, _("Elevation"), 35.0)
    description (_("Elevation angle (degrees). This appears to rotate the brightest pixels."))
    value_range (7, 70)
    ui_meta ("unit", "degree")

property_int (depth, _("Depth and or detail"), 24)
    description (_("Brings out depth and or detail of the bevel depending on the blend mode"))
    value_range (8, 45)


property_double (sharpen, _("Sharpen Radius"), 0.0)
    description (_("Unsharp mask radius that helps the bevel apper to have depth and structure to it."))
    value_range (0, 5)
    ui_range (0, 5)
   ui_meta ("visible", "advanced_options")




#else

#define GEGL_OP_META
#define GEGL_OP_NAME     ringbevel
#define GEGL_OP_C_SOURCE ringbevel.c

#include "gegl-op.h"

typedef struct
{
 GeglNode *input;
 GeglNode *levels;
 GeglNode *erase; 
 GeglNode *s1;
 GeglNode *s2;
 GeglNode *s3;
 GeglNode *s4;   
 GeglNode *nop1;
 GeglNode *nop2;
 GeglNode *nop3;
 GeglNode *nop4;        
 GeglNode *bevel;
 GeglNode *behind;
 GeglNode *median;
 GeglNode *bevel2; 
 GeglNode *over; 
 GeglNode *idref; 
 GeglNode *fix; 
 GeglNode *output;
}State;


static void attach (GeglOperation *operation)
{
  GeglNode *gegl = operation->node;
  GeglProperties *o = GEGL_PROPERTIES (operation);

  State *state = o->user_data = g_malloc0 (sizeof (State));


  state->input    = gegl_node_get_input_proxy (gegl, "input");
  state->output   = gegl_node_get_output_proxy (gegl, "output");


  state->bevel = gegl_node_new_child (gegl,
                                  "operation", "gegl:custom-bevel", "depth", 4.0, "gaus", 1.0, 
                                  NULL);


  state->bevel2 = gegl_node_new_child (gegl,
                                  "operation", "gegl:custom-bevel", "depth", 1.0, "gaus", 1.0, 
                                  NULL);

  state->erase = gegl_node_new_child (gegl,
                                  "operation", "gegl:dst-out", 
                                  NULL);

  state->behind = gegl_node_new_child (gegl,
                                  "operation", "gegl:dst-over", 
                                  NULL);

  state->idref = gegl_node_new_child (gegl,
                                  "operation", "gegl:nop", 
                                  NULL);

  state->nop1 = gegl_node_new_child (gegl,
                                  "operation", "gegl:nop", 
                                  NULL);

  state->nop2 = gegl_node_new_child (gegl,
                                  "operation", "gegl:nop", 
                                  NULL);

  state->nop3 = gegl_node_new_child (gegl,
                                  "operation", "gegl:nop", 
                                  NULL);

  state->nop4 = gegl_node_new_child (gegl,
                                  "operation", "gegl:nop", 
                                  NULL);

  state->fix = gegl_node_new_child (gegl,
                                  "operation", "gegl:median-blur", "radius", 0, 
                                  NULL);

  state->levels = gegl_node_new_child (gegl,
                                  "operation", "gegl:levels",
                                  NULL);

  state->s1 = gegl_node_new_child (gegl,
                                  "operation", "gegl:ssg", "radius", 0, "stroke", 7.0,  
                                  NULL);

  state->s2 = gegl_node_new_child (gegl,
                                  "operation", "gegl:ssg", "radius", 0, "stroke", 7.0,  
                                  NULL);

  state->s3 = gegl_node_new_child (gegl,
                                  "operation", "gegl:ssg", "radius", 0, "stroke", 7.0,  
                                  NULL);

  state->s4 = gegl_node_new_child (gegl,
                                  "operation", "gegl:ssg", "radius", 0, "stroke", 7.0,  
                                  NULL);


  state->median = gegl_node_new_child (gegl,
                                  "operation", "gegl:median-blur", "radius", 5, "alpha-percentile", 0.0,  
                                  NULL);


static void update_graph (GeglOperation *operation)
{
  GeglProperties *o = GEGL_PROPERTIES (operation);
  State *state = o->user_data;
  if (!state) return;

    gegl_operation_meta_redirect (operation, "azimuth", state->bevel, "azimuth");
}

switch (o->rings) {
        break;
    case onering:
  gegl_node_link_many (state->input, state->levels, state->idref, state->erase, state->bevel, state->behind, state->output, NULL);
  gegl_node_link_many (state->idref, state->median, state->s1, state->fix, NULL);
  gegl_node_connect_from (state->erase, "aux", state->fix, "output");
  gegl_node_link_many (state->idref, state->bevel2,   NULL);
  gegl_node_connect_from (state->behind, "aux", state->bevel, "output");
        break;
    case tworings:
  gegl_node_link_many (state->input, state->levels, state->idref, state->erase, state->bevel, state->behind, state->output, NULL);
  gegl_node_link_many (state->idref, state->median, state->s1, state->s2, state->fix, NULL);
  gegl_node_connect_from (state->erase, "aux", state->fix, "output");
  gegl_node_link_many (state->idref, state->bevel2,   NULL);
  gegl_node_connect_from (state->behind, "aux", state->bevel, "output");
        break;
    case threerings:
  gegl_node_link_many (state->input, state->levels, state->idref, state->erase, state->bevel, state->behind, state->output, NULL);
  gegl_node_link_many (state->idref, state->median, state->s1, state->s2, state->s3, state->fix, NULL);
  gegl_node_connect_from (state->erase, "aux", state->fix, "output");
  gegl_node_link_many (state->idref, state->bevel2,   NULL);
  gegl_node_connect_from (state->behind, "aux", state->bevel, "output");
        break;
    case fourrings:
  gegl_node_link_many (state->input, state->levels, state->idref, state->erase, state->bevel, state->behind, state->output, NULL);
  gegl_node_link_many (state->idref, state->median, state->s1, state->s2, state->s3, state->s4, state->fix, NULL);
  gegl_node_connect_from (state->erase, "aux", state->fix, "output");
  gegl_node_link_many (state->idref, state->bevel2,   NULL);
  gegl_node_connect_from (state->behind, "aux", state->bevel, "output");
    }
}





static void
gegl_op_class_init (GeglOpClass *klass)
{
  GeglOperationClass *operation_class;
GeglOperationMetaClass *operation_meta_class = GEGL_OPERATION_META_CLASS (klass);
  operation_class = GEGL_OPERATION_CLASS (klass);

  operation_meta_class->update = update_graph;
operation_class->attach = attach;

  gegl_operation_class_set_keys (operation_class,
    "name",        "lb:ringbevel",
    "title",       _("Ring Bevel"),
    "reference-hash", "r34jghavetheringrightherejfj3",
    "description", _("Make a Ringed Bevel effect"),
    "gimp:menu-path", "<Image>/Filters/Text Styling",
    "gimp:menu-label", _("Ringed Bevel..."),
    NULL);
}

#endif
